@@ -2,11 +2,11 @@
 **  Copyright (c) 2004-2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dk-filter.c,v 1.180 2008/03/12 14:17:17 msk Exp $
+**  $Id: dk-filter.c,v 1.183 2008/09/04 18:32:36 msk Exp $
 */
 
 #ifndef lint
-static char dk_filter_c_id[] = "@(#)$Id: dk-filter.c,v 1.180 2008/03/12 14:17:17 msk Exp $";
+static char dk_filter_c_id[] = "@(#)$Id: dk-filter.c,v 1.183 2008/09/04 18:32:36 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -74,6 +74,7 @@ static char dk_filter_c_id[] = "@(#)$Id: dk-filter.c,v 1.180 2008/03/12 14:17:17
 
 /* dk-filter includes */
 #include "dk-filter.h"
+#include "dk-crypto.h"
 #include "util.h"
 
 #ifdef DEBUG
@@ -3684,10 +3685,11 @@ main(int argc, char **argv)
 		pw = getpwnam(become);
 		if (pw == NULL)
 		{
+			char *q;
 			uid_t uid;
 
-			uid = atoi(become);
-			if (uid != 0 && uid != LONG_MIN && uid != LONG_MAX)
+			uid = (uid_t) strtol(become, &q, 10);
+			if (*q == '\0')
 				pw = getpwuid(uid);
 			if (pw == NULL)
 			{
@@ -4094,7 +4096,15 @@ main(int argc, char **argv)
 		}
 	}
 
-	ERR_load_crypto_strings();
+	status = dkf_crypto_init();
+	if (status != 0)
+	{
+		if (dolog)
+			syslog(LOG_ERR, "can't initialize OpenSSL library");
+
+		dkf_zapkey();
+		return EX_UNAVAILABLE;
+	}
 
 	/* initialize the DomainKeys package */
 	libdk = dk_init(NULL, NULL);
@@ -4200,6 +4210,7 @@ main(int argc, char **argv)
 			       popdbfile);
 
 			dkf_zapkey();
+			dkf_crypto_free();
 			return EX_UNAVAILABLE;
 		}
 	}
@@ -4233,6 +4244,8 @@ main(int argc, char **argv)
 	}
 #endif /* POPAUTH */
 
+	dkf_crypto_free();
+
 	dkf_zapkey();
 
 	dk_close(libdk);
