@@ -1,12 +1,12 @@
 /*
-**  Copyright (c) 2004-2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2004-2009 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  $Id: dk.c,v 1.186 2008/03/09 15:48:02 msk Exp $
+**  $Id: dk.c,v 1.192 2009/02/04 17:52:47 msk Exp $
 */
 
 #ifndef lint
-static char dk_c_id[] = "@(#)$Id: dk.c,v 1.186 2008/03/09 15:48:02 msk Exp $";
+static char dk_c_id[] = "@(#)$Id: dk.c,v 1.192 2009/02/04 17:52:47 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -1921,23 +1921,24 @@ dk_canon(DK *dk, unsigned char *buf, size_t buflen)
 }
 
 /*
-**  DK_VERROR -- log an error into a DK handle (varargs version)
+**  DK_ERROR -- log an error into a DK handle
 **
 **  Parameters:
 **  	dk -- DK context in which this is performed
 **  	format -- format to apply
-**  	va -- argument list
+**  	... -- arguments
 **
 **  Return value:
 **  	None.
 */
 
-static void
-dk_verror(DK *dk, const char *format, va_list va)
+void
+dk_error(DK *dk, const char *format, ...)
 {
 	int flen;
 	int saverr;
 	char *newdk;
+	va_list va;
 
 	assert(dk != NULL);
 	assert(format != NULL);
@@ -1958,7 +1959,9 @@ dk_verror(DK *dk, const char *format, va_list va)
 
 	for (;;)
 	{
+		va_start(va, format);
 		flen = vsnprintf(dk->dk_error, dk->dk_errlen, format, va);
+		va_end(va);
 
 		/* compensate for broken vsnprintf() implementations */
 		if (flen == -1)
@@ -1988,31 +1991,6 @@ dk_verror(DK *dk, const char *format, va_list va)
 	errno = saverr;
 }
 
-/*
-**  DK_ERROR -- log an error into a DK handle
-**
-**  Parameters:
-**  	dk -- DK context in which this is performed
-**  	format -- format to apply
-**  	... -- arguments
-**
-**  Return value:
-**  	None.
-*/
-
-static void
-dk_error(DK *dk, const char *format, ...)
-{
-	va_list va;
-
-	assert(dk != NULL);
-	assert(format != NULL);
-
-	va_start(va, format);
-	dk_verror(dk, format, va);
-	va_end(va);
-}
-
 /*
 **  ===== PUBLIC SECTION
 */
@@ -2826,13 +2804,6 @@ dk_body(DK *dk, u_char *buf, size_t len)
 					}
 					else
 					{
-						for (c = 0;
-						     c < dk->dk_blanks;
-						     c++)
-							DK_CANON(dk, CRLF, 2);
-
-						dk->dk_blanks = 0;
-
 						DK_CANON(dk, wrote, wlen);
 						DK_CANON(dk, CRLF, 2);
 
@@ -2851,7 +2822,21 @@ dk_body(DK *dk, u_char *buf, size_t len)
 
 		/* write what's left */
 		if (wlen > 0)
+		{
+			if (dk->dk_blanks > 0)
+			{
+				int c;
+
+				for (c = 0;
+				     c < dk->dk_blanks;
+				     c++)
+					DK_CANON(dk, CRLF, 2);
+
+				dk->dk_blanks = 0;
+			}
+
 			DK_CANON(dk, wrote, wlen);
+		}
 
 		break;
 
@@ -2899,6 +2884,18 @@ dk_body(DK *dk, u_char *buf, size_t len)
 			{
 				if (lidx == sizeof lbuf)
 				{
+					if (dk->dk_blanks > 0)
+					{
+						int c;
+
+						for (c = 0;
+						     c < dk->dk_blanks;
+						     c++)
+							DK_CANON(dk, CRLF, 2);
+
+						dk->dk_blanks = 0;
+					}
+
 					DK_CANON(dk, lbuf, lidx);
 					lidx = 0;
 				}
@@ -2911,7 +2908,21 @@ dk_body(DK *dk, u_char *buf, size_t len)
 		}
 
 		if (lidx > 0)
+		{
+			if (dk->dk_blanks > 0)
+			{
+				int c;
+
+				for (c = 0;
+				     c < dk->dk_blanks;
+				     c++)
+					DK_CANON(dk, CRLF, 2);
+
+				dk->dk_blanks = 0;
+			}
+
 			DK_CANON(dk, lbuf, lidx);
+		}
 
 		break;
 
@@ -2951,6 +2962,10 @@ dk_eom(DK *dk, DK_FLAGS *dkf)
 	{
 		int status;
 
+#ifdef _FFR_HASH_BUFFERING
+		dk_canonbuffer(dk, NULL, 0);
+#endif /* _FFR_HASH_BUFFERING */
+
 		/* no sender header was found below the signature */
 		if (dk->dk_skipbody && dk->dk_processing)
 		{
@@ -2989,10 +3004,6 @@ dk_eom(DK *dk, DK_FLAGS *dkf)
 			return DK_STAT_NORESOURCE;
 		}
 
-#ifdef _FFR_HASH_BUFFERING
-		dk_canonbuffer(dk, NULL, 0);
-#endif /* _FFR_HASH_BUFFERING */
-
 		/* set up the RSA object */
 		sha1->sha1_rsa = EVP_PKEY_get1_RSA(sha1->sha1_pkey);
 		if (sha1->sha1_rsa == NULL)
